﻿/*****************************************************************************  
* Description        : RSW-ECAT-Master : DO ﾌﾟﾛｸﾞﾗﾑｻﾝﾌﾟﾙ                        
\*****************************************************************************/
using System;
using System.Threading;
using System.Runtime.InteropServices;
using RSWECAT_dotNet;


/*****************************************************************************  
* main 処理                                                                     
\*****************************************************************************/
class Program
{
    static void Main(string[] args)
    {
        EcatDIOProgram test = new EcatDIOProgram();
        test.StartTestDIO();                        // 処理実行
        Console.ReadLine();                         // 待機
        test.StopTestDIO();                         // 処理実行
    }
}

/*****************************************************************************  
* DO 処理                                                                       
\*****************************************************************************/
class EcatDIOProgram
{
    EcatHandler hEcat = null;
    ushort usReqState = 0, usNowState = 0;
    SLAVE_DETAIL stSlave;
    uint uiCount = 0;
            
    /***************************************************************************** 
    * 開始処理
    \*****************************************************************************/
    public void StartTestDIO()
    {
        try
        {
            // RSW-ECAT初期化(RSW-ECAT API ｺｰﾙ)
            // システムの初期化完了を待機するため複数回コールします。
            for (int i = 0; i < 5; i++)
            {
                try { hEcat = new EcatHandler(); }
                catch (EcatException er)
                {
                    if (i >= 5) { throw er; }
                }
                if (hEcat != null) { break; }
            }

            // Masterｽﾃｰﾄﾁｪﾝｼﾞ要求 (RSW-ECAT API ｺｰﾙ)                           
            // MST_OPERATIONAL : ｵﾍﾟﾚｰｼｮﾅﾙへ遷移                                
            hEcat.WEhRqState(EcatHandler.MST_OPERATIONAL);

            // ｵﾍﾟﾚｰｼｮﾅﾙへ遷移したかどうかの確認処理                            
            while (true)
            {
                // 現在の Masterｽﾃｰﾄ状態取得 (RSW-ECAT API ｺｰﾙ)                 
                hEcat.WEhGetState(ref usReqState, ref usNowState);

                // ｵﾍﾟﾚｰｼｮﾅﾙへ遷移完了かをﾁｪｯｸ                                  
                if (usNowState == EcatHandler.MST_OPERATIONAL)
                {
                    break;
                }
                // ｽﾃｰﾄﾁｪﾝｼﾞ待機用ｽﾘｰﾌﾟ                                         
                Thread.Sleep(500);
            }

            // ここからｽﾚｰﾌﾞへのｱｸｾｽ処理 

            // ｽﾚｰﾌﾞ検索                                                        
            stSlave = new SLAVE_DETAIL();
            stSlave.fnAryClear();                                       // 必ず行う
            stSlave.uiSize = (uint)Marshal.SizeOf(stSlave);
            stSlave.uiVendorID = 131;
            stSlave.uiProductCode = 50;
            stSlave.uiInstance = 0;

            hEcat.WEhFindSlave(ref stSlave);

            // Callbackの登録
            hEcat.WEhSetEventCallBack(CallBack);
        }
        catch (EcatException er)
        {
            Console.WriteLine("<< EcatException >>");
            Console.WriteLine(" - ApiReturn:0x{0:x8}", er.uiApiReturn);
            Console.WriteLine(" - DotNetErr:0x{0:x8}", er.uiDotNetErr);
            Console.WriteLine(" - IntrErr  :0x{0:x8}", er.uiIntrErr);
            Console.WriteLine("\n( InnerExp info )\n" + er.expDotNet.ToString());
            Console.WriteLine("program end.");

            // RSW-ECAT 終了処理 (RSW-ECAT API ｺｰﾙ)                                
            if (hEcat != null)
            {
                hEcat.WEhClose();
            }
        }
    }

    /***************************************************************************** 
    * イベント発生時にコールされるメソッド
    \*****************************************************************************/
    private void CallBack(object sender, EcatHandler.EcatEventArgs e)
    {
        try
        {
            // イベント処理
            switch (e.Type)
            {
                // サイクリックイベント処理
                case EcatHandler.EcatEventType.Cyclic:
                    uiCount++;                                                  // イベント数のカウント

                    if (200 < uiCount)
                    {
                        uiCount = 0;

                        // ﾙｰﾌﾟ毎にON/OFF変換要求
                        if (0 != hEcat.WEhReadbackByte(stSlave.uiViosOutBaseOffset))
                        {
                            hEcat.WEhWriteByte(stSlave.uiViosOutBaseOffset, 0x00);
                        }
                        else
                        {
                            hEcat.WEhWriteByte(stSlave.uiViosOutBaseOffset, 0xFF);
                        }
                    }
                    break;

                // 診断イベント
                case EcatHandler.EcatEventType.DiagEvent:
                    break;

                // イベント待機処理にて例外発生
                case EcatHandler.EcatEventType.Exception:
                    hEcat.WEhRqState(EcatHandler.MST_INIT);                     // INITへ（サイクリックイベントの停止）
                    hEcat.WEhClose();                     // INITへ（サイクリックイベントの停止）
                    return;
            }
        }
        catch (EcatException er)
        {
            // エラー処理
        }
    }

        
    /***************************************************************************** 
    * 終了処理
    \*****************************************************************************/
    public void StopTestDIO() 
    {
        try
        {
            // 回線クローズ(RSW-ECAT API ｺｰﾙ)
            hEcat.WEhClose();
        }
        catch (EcatException er)
        {
            Console.WriteLine("<< EcatException >>");
            Console.WriteLine(" - ApiReturn:0x{0:x8}", er.uiApiReturn);
            Console.WriteLine(" - DotNetErr:0x{0:x8}", er.uiDotNetErr);
            Console.WriteLine(" - IntrErr  :0x{0:x8}", er.uiIntrErr);
            Console.WriteLine("\n( InnerExp info )\n" + er.expDotNet.ToString());
            Console.WriteLine("program end.");

            // RSW-ECAT 終了処理 (RSW-ECAT API ｺｰﾙ)                                
            if (hEcat != null)
            {
                hEcat.WEhClose();
            }
        }
    }

}
